package de.opalpiraten.scenepainter.gui.render;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.awt.geom.*;
import javax.imageio.*;
import java.awt.image.*;
import java.net.*;
import de.opalpiraten.scenepainter.sys.*;
import com.threerings.media.image.*;

/**
 * The PictureSceneElement is constructed from a PNG file.
 */
public class PictureSceneElement extends SceneElement {

  /**
   * The resourcename (filename without path and .png suffix)
   */
  private String resource;
  
  /**
   * For recoloring
   */
  private Hashtable zations = new Hashtable();
  
  /**
   * Set the name of the image
   * @param r of the resource, as seen by the project. 
   */
  public void setResource(String r) {
    resource=r;
    setName(r);
    cache=null;
  }
  
  /**
   * Query Colorizations to apply. The key eleemnt is the colorclassname, the
   * value the colorname (both strings)
   */
  public Hashtable getZations() {
    return zations;
  }
  
  /**
   * Set recolorings. Key is the colorclassname, value is the colorname
   */
  public void setZations(Hashtable z) {
    zations=z;
    cache=null;
  }
  
  /**
   * Query the name of resource for loading the image
   */
  public String getResource() {
    return resource;
  }
  
  public BufferedImage buildCache() {
    BufferedImage tmp = null;
    try {
      tmp=project.loadPicture(resource);
      if (tmp==null) throw new NullPointerException();
      
      Colorization cz[] = new Colorization[0];
      if (zations!=null) {
        cz = new Colorization[zations.size()];
        int i=0;
        Enumeration e =zations.keys();
        while(e.hasMoreElements()) {
          String clazz = (String)e.nextElement();
          String color = (String)zations.get(clazz);
          cz[i] = project.getColorPository().getColorization(clazz,color);
          i++;
        }
        if (cz.length!=0) tmp = ImageUtil.recolorImage(tmp, cz);
      }
   
    }
    catch (Exception e) {
      // Construct a dummy image, showing the resourcename as an error
      tmp = new BufferedImage(1,1,BufferedImage.TYPE_INT_ARGB);
      Graphics g = tmp.createGraphics();
      FontMetrics fm = g.getFontMetrics();
      Rectangle2D bounds = fm.getStringBounds(resource,g);
      tmp = new BufferedImage((int)(bounds.getWidth()+2),(int)(bounds.getHeight()+2),BufferedImage.TYPE_INT_ARGB);
      g = tmp.createGraphics();
      g.setColor(Color.RED.brighter());
      g.fillRect(0,0,(int)(bounds.getWidth()+2),(int)(bounds.getHeight()+2) );
      g.setColor(Color.BLACK);
      g.drawString(resource,1,1+fm.getMaxAscent());
      return tmp;
    }
    
    
    BufferedImage ret = new BufferedImage(tmp.getWidth(),tmp.getHeight(),BufferedImage.TYPE_INT_ARGB);
    Graphics2D g = ret.createGraphics();
    g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,alpha));
    g.drawImage(tmp,0,0,null);
    return ret;
  }
}
